% Continuous-Time Kalman Filter
% Author: Randal Allen (2010)

clear all;
clc;
close all

% Radar range measurement error sigma_range = 100 [m]
Rc = 100^2;

% State vector x = [ r rdot rddot ]
% xhat(0) = E[x(0)]
% Initial range = 200000 m
% Initial velocity = -5000 m/s
% Acceleration = -9.81 m/s/s
xhat0 = [ 200000 -5000 -9.81 ]';

% Covariance matrix
% P(0) = E[(x(0)-xhat(0)) (x(0)-xhat(0))']
% Initial conditions assumed to have significant error
P0 = [ 9999  0  0;  0  9999  0;  0  0  9999 ];

% Process noise matrix
% Range = 1 [m]
% Velocity = 0.1 [m/s]
% Acceleration = 0.1 [m/s/s]
Qc = [ 1^2  0  0;  0  0.1^2  0;  0  0  0.1^2 ];

% Kinematics
% r = (1/2)*a0*t^2 + v0*t + r0
% v = a0*t + v0
% a = a0

% State-space representation of kinematics
A = [ 0  1  0;  0  0  1;  0  0  0 ];
B = [ 0  0  0 ]';
% Observing range
C = [ 1  0  0 ];
D = 0;

% Process at a rate of 10 Hz for 30 sec
tfinal = 30;
dt = 0.1;
P = P0;
xhat = xhat0;

% Run the simulation
options=simset('Solver','ode4','FixedStep',dt);
sim('CTKF_Model',tfinal,options);

% Plot range and estimate for comparison
figure(100)
plot(yout.time,yout.signals.values,'b','LinewIdth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(yhatout.time,yhatout.signals.values(:,1),'b','LinewIdth',1.3)
ylabel('Range [m]','FontSize',13)
text(3,2.05e5,'Range (R) and Range Estimate (R) [m] versus Time [s]','FontSize',12)
text(19.1,2.11e5,'\bf\wedge')
text(11,1.235e5,'R','FontSize',13)
plot([12 15],[1.235e5 1.235e5],'k','LineWidth',1.1)
text(10,1.635e5,'R','FontSize',13)
plot([7.25 9.5],[1.635e5 1.635e5],'k','LineWidth',1.1)
text(10.1,1.7e5','\bf\wedge')


% %Plot velocity and estimate for comparison
figure(200)
plot(vout.time,vout.signals.values,'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(yhatout.time,yhatout.signals.values(:,2),'r','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Velocity and Velocity Estimate [m/s]','FontSize',13)
text(0,-4880,'Velocity (V)and Velocity Estimate (V) [m/s] versus Time [s]','FontSize',12)
text(16.97,-4865,'\bf\wedge')
axis([0 30 -5300 -4900])
text(1,-5140,'velocity estimate (V)','FontSize',13)
text(10.42,-5122,'\bf\wedge')
plot([2.3 2.3],[-5125 -5070],'k','LineWidth',1.2)
text(6,-4990,'velocity (V)','FontSize',13)
plot([2.5 5],[-4992 -4992],'k','LineWidth',1.2)
plot([2 2.5],[-5018 -4992],'k','LineWidth',1.2)


% Plot acceleration and estimate for comparison
figure(300)
plot(aout.time,aout.signals.values,'r','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(yhatout.time,yhatout.signals.values(:,3),'b','LineWidth',1.3)
xlabel('Time [s]')
ylabel('Acceleration and Acceleration Estimate [m/s^2]','FontSize',13)
text(0,1.5,'Grav. Acc. and Acceleration Estimate (A) [m/s^2] versus Time [s]','FontSize',12)
text(19.0,2.7,'\bf\wedge')
axis([0 30 -35 0])
text(3.5,-24.0,'acceleration estimate (A)','FontSize',13)
text(15.4,-22.65,'\bf\wedge')

text(10,-14,'gravitational acceleration','FontSize',13)
plot([7 9.5],[-14 -14],'k','LineWidth',1.2)
plot([7 7],[-14 -10],'k','LineWidth',1.2)

for n=1:301
	sqrtp11(n) = sqrtp.signals.values(1,1,n);
	sqrtp22(n) = sqrtp.signals.values(2,2,n);
	sqrtp33(n) = sqrtp.signals.values(3,3,n);
end

% Plot range error
figure(400)
plot(yout.time,yout.signals.values-yhatout.signals.values(:,1),'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(sqrtp.time,sqrtp11,'k--','LineWidth',1.3)
plot(sqrtp.time,-sqrtp11,'k--','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Range Error [m]','FontSize',13)
title('Range Error [m] Compared with Theoretical Bounds')
axis([0 30 -150 150])
text(6.5,35,'range error','FontSize',13)
text(15,107,'upper bound','FontSize',13)
% text(15,-94,'lower bound','FontSize',13)

% Plot velocity error
figure(500)
plot(vout.time,vout.signals.values-yhatout.signals.values(:,2),'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(sqrtp.time,sqrtp22,'k--','LineWidth',1.3)
plot(sqrtp.time,-sqrtp22,'k--','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Velocity Error [m/s]','FontSize',13)
title('Velocity Error [m/s] Compared with Theoretical Bounds')
axis([0 30 -150 150])
text(0.8,-35,'velocity error','FontSize',13)
text(12,57,'upper bound','FontSize',13)
text(12,-43,'lower bound','FontSize',13)
