% Discrete-Time Kalman Filter
% Author: Randal Allen (2010)

clear all
clc
close all

% Radar range measurement error sigma_range = 100 [m]
Rc = 100^2;

% State vector x = [ r rdot rddot ]
% xhat(0) = E[x(0)]
% Assume unknown
xhat0 = [ 200000 -5000 -9.81 ]';

% Covariance matrix
% P(0) = E[(x(0)-xhat(0)) (x(0)-xhat(0))']
% Initial conditions assumed to have significant error
P0 = [ 1e10  0  0;  0  1e10  0;  0  0  1e10 ];

% Kinematics
% r = (1/2)*a0*t^2 + v0*t + r0
% v = a0*t + v0
% a = a0

% State-space representation of kinematics
A = [ 0  1  0;  0  0  1;  0  0  0 ];
B = [ 0  0  0 ]';
% Observing range
C = [ 1  0  0 ];
D = 0;

% Discretized state-space representation of kinematics
dt = 0.1;
Rk = Rc/dt;
Fk = [ 1  dt  dt*dt/2;  0  1  dt;  0  0  1 ];
Hk = C;
I = eye(3);
Qk = 0 * [ dt^5/20  dt^4/8  dt^3/6;  dt^4/8  dt^3/3  dt^2/2;  dt^3/6  dt^2/2  dt ];

% Process at a rate of 10 Hz for 30 sec
t = 0;
tfinal = 30;
Pm0 = P0;
xm0 = xhat0;

% Run the simulation
options=simset('Solver','FixedStepDiscrete','FixedStep',dt);
sim('DTKF_Model',tfinal,options);

% Plot range and estimate for comparison
del = 5;
figure(100)
plot(yout.time,yout.signals.values,'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(xmout.time(1:del:end),xmout.signals.values((1:del:end),1),'r.','MarkerSize',10)
xlabel('Time [s]','FontSize',13)
ylabel('Range and Range Estimate [m]','FontSize',13)
text(3,2.05e5,'Range (R) and Range Estimate (R) [m] versus Time [s]','FontSize',12)
text(19.15,2.12e5,'\bf\wedge')
text(7.0,1.23e5,'R','FontSize',13)
plot([5 6],[1.22e5 1.22e5],'b','LineWidth',1.3)
text(7.0,1.03e5,'R','FontSize',13)
text(7.07,1.10e5,'\bf\wedge')
plot(5.5,1.03e5,'.r','MarkerSize',10)


% Plot velocity and estimate for comparison
figure(200)
del=5;
plot(vout.time(1:del:end),vout.signals.values(1:del:end),'b','LineWidth',1.3)
axis([0 30 -5320 -4880])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(xmout.time(1:del:end),xmout.signals.values((1:del:end),2),'.r','MarkerSize',10)
xlabel('Time [s]','FontSize',13)
ylabel('Velocity and Velocity Estimate [m/s]','FontSize',13)
text(0.5,-4865,'Velocity (V) and Velocity Estimate (V) [m/s] versus Time [s]','FontSize',12)
text(17.82,-4846,'\bf\wedge')
text(15.0,-4940,'V','FontSize',13)
plot([13 14],[-4940 -4940],'b','LineWidth',1.3)
text(15.0,-4990,'V','FontSize',13)
text(15.1,-4970,'\bf\wedge')
plot(13.5,-4990,'.r','MarkerSize',10)


% % Plot acceleration and estimate for comparison
figure(300)
del=5;
plot(aout.time,aout.signals.values,'b','LineWidth',1.3)
axis([0 30 -15 25])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(xmout.time(1:del:end),xmout.signals.values((1:del:end),3),'.r','MarkerSize',10)
xlabel('Time [s]','FontSize',13)
ylabel('Acceleration and Acceleration Estimate [m/s^2]','FontSize',13)
text(-1,27.3,'Grav Acc (A) and Acceleration Estimate (A) [m/s^2] versus Time [s]','FontSize',12)
text(19.30,28.66,'\bf\wedge')
text(14,15.90,'gravitational acceleration (A)','FontSize',13)
plot([12 13],[15.90 15.90],'b','LineWidth',1.3)
text(14,10.83,'acceleration estimate (A)','FontSize',13)
text(25.9,12.68,'\bf\wedge')
plot(12.5,10.81','.r','MarkerSize',10)

for n=1:301
	sqrtp11(n) = sqrtPmout.signals.values(1,1,n);
	sqrtp22(n) = sqrtPmout.signals.values(2,2,n);
	sqrtp33(n) = sqrtPmout.signals.values(3,3,n);
end

% % Plot range error
figure(400)
del=5;
plot(yout.time(1:del:end),yout.signals.values(1:del:end)-xmout.signals.values((1:del:end),1),'.b','MarkerSize',10)
axis([0 30 -200 200])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(sqrtPmout.time,sqrtp11,'--k','LineWidth',1.3)
plot(sqrtPmout.time,-sqrtp11,'--k','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Range Error [m]','FontSize',13)
title('Range Error [m] Compared with Theoretical Bounds')
text(12,110,'upper bound','FontSize',13)
text(12,-110,'lower bound','FontSize',13)
text(4,40,'range error','FontSize',13)

% % Plot velocity error
figure(500)
del=5;
plot(vout.time(1:del:end),vout.signals.values(1:del:end)-xmout.signals.values((1:del:end),2),'.b','MarkerSize',10)
axis([0 30 -200 200])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(sqrtPmout.time,sqrtp22,'--k','LineWidth',1.3)
plot(sqrtPmout.time,-sqrtp22,'--k','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Veloocity Error [m/s]','FontSize',13)
title('Velocity Error [m/s] Compared with Theoretical Bounds')
text(15,60,'upper bound','FontSize',13)
text(15,-60,'lower bound','FontSize',13)
text(3,10,'velocity error','FontSize',13)

