% Steady-State Continuous-Time Kalman Filter
% Author: Randal Allen (2010)

clear all 
clc 
close all

% Radar range measurement error sigma_range = 100 [m]
Rc = 100^2;

% State vector x = [ r rdot rddot ]
% xhat(0) = E[x(0)]
% Initial range = 200000 m
% Initial velocity = -5000 m/s
% Acceleration = -9.81 m/s/s
xhat0 = [ 200000 -5000 -9.81 ]';

% Covariance matrix
% P(0) = E[(x(0)-xhat(0)) (x(0)-xhat(0))']
% Initial conditions assumed to have significant error
P0 = [ 9999  0  0;  0  9999  0;  0  0  9999 ];

% Process noise matrix
% Range = 1 [m]
% Velocity = 0.1 [m/s]
% Acceleration = 0.1 [m/s/s]
Qc = [ 1^2  0  0;  0  0.1^2  0;  0  0  0.1^2 ];

% Kinematics
% r = (1/2)*a0*t^2 + v0*t + r0
% v = a0*t + v0
% a = a0

% State-space representation of kinematics
A = [ 0  1  0;  0  0  1;  0  0  0 ];
B = [ 0  0  0 ]';
% Observing range
C = [ 1  0  0 ];
D = 0;

% Steady-state covariance matrix
SSP = [ 458.26 10 0; 10 0.45826 0; 0 0 0 ];

% Process at a rate of 10 Hz for 30 sec
tfinal = 30;
dt = 0.1;
P = P0;
xhat = xhat0;

% Run the simulation
options=simset('Solver','ode4','FixedStep',dt);
sim('SSCTKF_Model',tfinal,options);

% Plot range and estimate for comparison
figure(100)
plot(yout.time,yout.signals.values,'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(yhatout.time,yhatout.signals.values(:,1),'r','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Range and Range Estimate [m]','FontSize',13)
text(3,2.05e5,'Range (R) and Range Estimate (R) [m] versus Time [s]','FontSize',12)
text(19.1,2.12e5,'\bf\wedge')
text(7,1.43e5,'R','FontSize',13)
plot([8.5 11.3],[1.43e5 1.43e5],'k','LineWidth',1.1)
text(18,1.23e5,'R','FontSize',13)
text(18.05,1.30e5,'\bf\wedge')
plot([15.2 17.5],[1.23e5 1.23e5],'k','LineWidth',1.1)

% Plot velocity and estimate for comparison
figure(200)
plot(vout.time,vout.signals.values,'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
plot(yhatout.time,yhatout.signals.values(:,2),'r','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Velocity and Velocity Estimate [m/s]','FontSize',13)
axis([0 30 -5300 -4980])

text(0.8,-4970,'Velocity (V) and Velocity Estimate (V) [m/s] versus Time [s]','FontSize',12)
text(18.20,-4956,'\bf\wedge')
text(5.5,-5091,'V','FontSize',13)
plot([6.5 9.3],[-5091 -5091],'k','LineWidth',1.1)
text(17.5,-5141,'V','FontSize',13)
text(17.55,-5128,'\bf\wedge')
plot([14.3 17],[-5141 -5141],'k','LineWidth',1.1)


% % Plot acceleration and estimate for comparison
figure(300)
plot(aout.time,aout.signals.values,'b','LineWidth',1.3)
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
set(gca,'YTick',[-15:2:-5])
plot(yhatout.time,yhatout.signals.values(:,3),'r','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Acceleration and Acceleration Estimate [m/s^2]','FontSize',13)
text(-0.7,-4.50,'Grav. Acc. and Acceleration Estimate (A) [m/s^2] versus Time [s]','FontSize',12)
text(18.4,-4.20,'\bf\wedge')
axis([0 30 -15 -5])
text(5,-10.75,'gravitational acceleration','FontSize',13)
plot([2 4],[-10.75 -10.75],'k','LineWidth',1.1)
plot([2 2],[-10.75 -9.8],'k','LineWidth',1.1)
text(15,-8.75,'acceleration estimate, (A)','FontSize',13)
plot([12 14],[-8.75 -8.75],'k','LineWidth',1.1)
plot([12 12],[-8.75 -9.8],'k','LineWidth',1.3)
text(27.25,-8.35','\bf\wedge')

for n=1:301
	sqrtp11(n) = sqrtp.signals.values(1,1,n);
	sqrtp22(n) = sqrtp.signals.values(2,2,n);
	sqrtp33(n) = sqrtp.signals.values(3,3,n);
end

% % Plot range error
figure(400)
plot(yout.time,yout.signals.values-yhatout.signals.values(:,1),'b','LineWidth',1.3)
axis([0 30 -25 25])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
set(gca,'YTick',[-25:5:25])
plot(sqrtp.time,sqrtp11,'--k','LineWidth',1.3)
plot(sqrtp.time,-sqrtp11,'--k','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Range Error [m]','FontSize',13)
title('Range Error [m] Compared with Theoretical Bounds')
text(10,2,'range error','FontSize',13)
text(10,19,'upper bound','FontSize',13)
text(10,-18.5,'lower bound','FontSize',13)

% % Plot velocity error
figure(500)
plot(vout.time,vout.signals.values-yhatout.signals.values(:,2),'b','LineWidth',1.3)
axis([0 30 -1 1])
grid on
hold on
set(gca,'FontSize',11)
set(gca,'XTick',[0:5:30])
set(gca,'YTick',[-1:0.25:1])
plot(sqrtp.time,sqrtp22,'--k','LineWidth',1.3)
plot(sqrtp.time,-sqrtp22,'--k','LineWidth',1.3)
xlabel('Time [s]','FontSize',13)
ylabel('Veloocity Error [m/s]','FontSize',13)
title('Veloocity Error [m/s] Compared with Theoretical Bounds')
text(10,0.08,'velocity error','FontSize',13)
text(10,0.78,'upper bound','FontSize',13)
text(10,-0.76,'lower bound','FontSize',13)

