% m-file Ch8_Multi_Rate_Integ.n
% calls Simulink model aircraft_pitch.mdl
% plots Figures 8.47, Figures 8.49 - 8.**

close all, clear all, clc

global A B Tf theta_com_amp tau_com

%%%%% airframe parameters %%%%%
wn=5; % natural frequency (rad/sec)
KA=10;
tauA=0.8;
zeta=0.2;

%%%%% control surface actuator parameters %%%%%
Kact=500;
Ce=0.08;
tau=0.01;
Kc=Kact*Ce/(tau^2);
z=1/Ce;
p1=1/tau;
p2=p1;
J=1;

%%%%% pitch input command parameters %%%%%
theta_com_amp_deg=5;
theta_com_amp=theta_com_amp_deg*pi/180;

K=0.1625;
Kq=0.2;

%%%%% State Eq matrices A, B, C, D %%%%%
wnsq=wn^2;
%%%%% Coefficient matrix %%%%%
A(7,7)=0;
A(1,2)=KA*wnsq;
A(1,3)=KA*wnsq*tauA;
A(2,3)=1;
A(3,2)=-wnsq;
A(3,3)=-2*zeta*wn;
A(3,4)=1;
A(4,5)=1;
A(5,6)=1;
A(6,1)=-Kc*K;
A(6,2)=-Kc*K*Kq*KA*wnsq;
A(6,3)=-Kc*K*Kq*KA*wnsq*tauA;
A(6,4)=-Kc;
A(6,6)=-p2;
A(6,7)=Kc*(z-p1);
A(7,1)=-K;
A(7,2)=-K*Kq*KA*wnsq;
A(7,3)=-K*Kq*KA*wnsq*tauA;
A(7,4)=-1;
A(7,7)=-p1;
%%%%% Input matrix %%%%%
B(1:7)=0;
B=B';
B(6)=Kc*K;
B(7)=K;
%%%%% Output matrix %%%%%
C(7,7)=0;
C(1,1)=1; % y1=x1=pitch (rad)
C(2,2)=1; % y2=x2
C(3,3)=1; % y3=x3
C(4,4)=1; % y4=x4=delta_e
C(5,5)=1; % y5=x5=delta_e_dot
C(6,6)=1; % y6=x6=delta_e_dbldot
C(7,7)=1; % y7=x7
%%%%% Direct transmission matrix %%%%%
D(1:7)=0;  
D=D';

%%%%% BEGIN MULTI-RATE INTEGRATION %%%%%

%%% slow states: u1=x1, u2=x2, u3=x3
%%% fast states: w1=x4, w2=x5, w3=x6, w4=x7

Ts=0.05; % slow time frame

%for Tf=[0.018 0.019] % fast time frame 
for Tf=0.018 % uncomment this line and comment out previous line after Figure 8.47 is done    
 
N=round(Ts/Tf); % frame ratio
tau_com=4*Tf; % time constant of exponential rise approximation of input step command
tfinal=5; % final simulation time (sec)
ifinal=round(tfinal/Ts); % total number of slow time frames
u(1:3)=0; % initialize slow states
u1(1)=u(1); u2(1)=u(2); u3(1)=u(3); % u1,u2,u3 are slow state arrays for plotting
w(1:4)=0; %initialize fast states
w1(1)=w(1); w2(1)=w(2); w3(1)=w(3); w4(1)=w(4); % w1,w2,w3,w4 are fast state arrays for plotting
w_old=w;
jcount=1; % index that counts the number of fast frames

for i=1:ifinal  % index number for slow time frame 
  
  ti=(i-1)*Ts;   
  
  %%% Steps 1-12 correspond to text
      
  %%% Step 1 - Compute k1=f(u,w), slow state derivatives at beginning of
  %%% slow frame
  [k1s(1),k1s(2),k1s(3)]=deriv_slow(u,w);  % k1s(1)=f1(u,w), k1s(2)=f2(u,w), k1s(3)=f3(u,w) 
  %k1s

  %%% Step 2 - Compute u_p=u+k1*(0.5*Ts)
  del=0.5*Ts; 
  u_p(1)=u(1)+k1s(1)*del;  % predicted u1 at midpoint of slow frame 
  u_p(2)=u(2)+k1s(2)*del;  % predicted u2 at midpoint of slow frame
  u_p(3)=u(3)+k1s(3)*del;  % predicted u3 at midpoint of slow frame
  %u_p

  %%% Step 4 - Integrate w1,w2,w3,w4 from t to t+0.5Ts using RK-4 
  jmid=round(N/2); % one half the number of fast frames
    for j=1:jmid
      jcount=jcount+1;
      tj=ti+(j-1)*Tf;  % absolute time at start of jth fast frame inside ith slow frame         
      u_interp(1)=u(1)+k1s(1)*(j-1)*Tf; % interpolated u1 at tj
      u_interp(2)=u(2)+k1s(2)*(j-1)*Tf; % interpolated u2 at tj
      u_interp(3)=u(3)+k1s(3)*(j-1)*Tf; % interpolated u3 at tj
      %u_interp
      w_new=RK4(tj,u_interp,w_old); % w1,w2,w3,w4 at tj+Tf, i.e. at end of fast frame starting at tj
      w1(jcount)=w_new(1); w2(jcount)=w_new(2); w3(jcount)=w_new(3); w4(jcount)=w_new(4);  % fast states w1,w2,w3,w4 at end of
                                                                                           % jth fast frame inside ith slow frame
      w_old=w_new;  % save new values of fast states for starting values for RK-4 integration of fast states over next fast frame
    end % of RK-4 integration of fast states w1,w2,w,3,w4 half way thru slow frame 
  
  %%% Step 5 - Compute k2=f(u_p,w_new), slow state derivatives at midpoint
  %%% of slow frame
  [k2s(1),k2s(2),k2s(3)]=deriv_slow(u_p,w_new);  
  %k2s
  
  %%% Step 6 - Compute u_pp=u+0.5*(k1+k2)*(0.5*TS), predicted slow states
  %%% at midpoint of slow frame
  u_pp(1)=u(1)+k2s(1)*del;  % second predicted u1 at midpoint of slow frame 
  u_pp(2)=u(2)+k2s(2)*del;  % second predicted u2 at midpoint of slow frame
  u_pp(3)=u(3)+k2s(3)*del;  % second predicted u3 at midpoint of slow frame
  %u_pp
  
  %%% Step 7 - Compute k3=f(u_pp,w_new), slow state derivatives at midpoint
  %%% of slow frame
  [k3s(1),k3s(2),k3s(3)]=deriv_slow(u_pp,w_new);  
  %k3s
  
  %%% Step 8 - Compute u_ppp=u+k3*Ts, predicted slow states at end of
  %%% slow frame
  u_ppp(1)=u(1)+k3s(1)*Ts;  % predicted u1 at end of slow frame 
  u_ppp(2)=u(2)+k3s(2)*Ts;  % predicted u2 at end of slow frame 
  u_ppp(3)=u(3)+k3s(3)*Ts;  % predicted u3 at end of slow frame 
  %u_ppp
  
  %%% Step 10 Integrate w from t+0.5Ts to t+Ts using RK-4 
    for j=1:jmid 
      jcount=jcount+1;
      tj=ti+(jmid+j-1)*Tf;  % absolute time at start of jth fast frame inside ith slow frame 
      m1=(u_ppp(1)-u_pp(1))/del;
      u_interp(1)=u_pp(1)+m1*(j-1)*Tf;
      m2=(u_ppp(2)-u_pp(2))/del;
      u_interp(2)=u_pp(2)+m2*(j-1)*Tf; 
      m3=(u_ppp(3)-u_pp(3))/del;
      u_interp(3)=u_pp(3)+m3*(j-1)*Tf;
      %u_interp
      w_new=RK4(tj,u_interp,w_old); 
      w1(jcount)=w_new(1); w2(jcount)=w_new(2); w3(jcount)=w_new(3); w4(jcount)=w_new(4);  % save fast states w1,w2,w3,w4 at end of
                                                                                           % jth fast frame inside ith slow frame
      w_old=w_new;  % save new values of fast states for starting values for RK-4 integration of fast states over next fast frame
    end % of RK-4 integration of fast states w1,w2,w3,w4 thru end of slow frame 
  
  %%% Step 11 - Compute k4=f(u_ppp,w_new), slow state derivatives at end of
  %%% slow frame
  [k4s(1),k4s(2),k4s(3)]=deriv_slow(u_ppp,w_new);  
  %k4s
  
  %%% Step 12 - Compute u_pppp=u+(Ts/6)*(k1+2k2+2k3+k4), updated slow
  %%% states at end of slow frame
  u_pppp(1)=u(1)+(Ts/6)*(k1s(1)+2*k2s(1)+2*k3s(1)+k4s(1));  % updated u1 at end of slow frame  
  u_pppp(2)=u(2)+(Ts/6)*(k1s(2)+2*k2s(2)+2*k3s(2)+k4s(2));  % updated u2 at end of slow frame 
  u_pppp(3)=u(3)+(Ts/6)*(k1s(3)+2*k2s(3)+2*k3s(3)+k4s(3));  % updated u3 at end of slow frame     
  
  u=[u_pppp(1) u_pppp(2) u_pppp(3)]; % store current values of slow states u1,u2,u3 in array u for next slow step iteration  
  %u
  u1(i+1)=u_pppp(1); u2(i+1)=u_pppp(2); u3(i+1)=u_pppp(3);  % save slow states u1,u2,u3 at end of ith slow frame
  ts(i+1)=i*Ts; % array containing beginning values of all slow time frames
          
end % for i

sim('aircraft_pitch')

figure(1) % plot Figure 8.47
t=pitch(:,1);
pitch=pitch(:,2);
theta=x1(:,2);

if Tf==0.018

subplot(2,2,1)
plot(t,pitch,'b','LineWidth',1.3)
grid on 
axis ([0 5 0 5.25])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:5])
text(-0.15,5.9,'Pitch \theta (deg), T = 0.018 s','FontSize',12)
ylabel('\theta (deg)','FontSize',13)

subplot(2,2,3)
plot(t,theta,'b','LineWidth',1.3)
grid on 
axis ([-0.2 5 0 5.25])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:5])
text(-0.45,5.82,'Pitch x_1 (deg), T = 0.018 s','FontSize',12)
xlabel('t  (s)','FontSize',13)
ylabel('x_1 (deg)','FontSize',13)

elseif Tf==0.019
    
subplot(2,2,2)
plot(t,pitch,'b','LineWidth',1.3)
grid on 
axis ([0 5 0 160])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:50:150])
text(-0.1,180,'Pitch \theta (deg), T = 0.019 s','FontSize',12)
ylabel('\theta (deg)','FontSize',13)

subplot(2,2,4)
plot(t,theta,'b','LineWidth',1.3)
grid on 
axis ([0 5 0 160])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:50:150])
text(-0.2,175,'Pitch x_1 (deg), T = 0.019 s','FontSize',12)
xlabel('t  (s)','FontSize',13)
ylabel('x_1 (deg)','FontSize',13)

end % if Tf 

end % for Tf
    
        
% % %%%%% Plot slow states u1,u2,u3

figure(2) % plot Figure 8.51

subplot(3,2,1)
x1=x1(:,2);
plot(t,x1,'b','LineWidth',1.3)
grid on
axis([0 tfinal 0 1.05*theta_com_amp_deg])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:2.5:5])
text(1.0,2.0,'x_1 - Simulink RK-4','FontSize',11)
text(0,6.2,'SIMULINK and Multi-rate Simulation of Slow States x_1, x_2 and x_3','FontSize',12)

subplot(3,2,2)
u1_deg=(180/pi)*u1; % convert u1 in rad to u1 in deg
plot(ts,u1_deg,'b','LineWidth',1.3)
grid on
axis([0 tfinal 0 1.05*theta_com_amp_deg])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:2.5:5])
text(0.45,1.25,'u_1 - Multirate RK-4/RK-4','FontSize',11)

subplot(3,2,3)
x2=x2(:,2);
plot(t,x2,'b','LineWidth',1.3)
grid on
% axis([0 tfinal 0 1.1*max(x2)])
axis([0 5 0 0.015])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:0.005:0.015])
text(1.25,0.011,'x_2 - Simulink RK-4','FontSize',11)

subplot(3,2,4)
u2_deg_sec=(180/pi)*u2; % convert u2 in rad/sec to u2 in deg/sec
plot(ts,u2_deg_sec,'b','LineWidth',1.3)
grid on
% axis([0 tfinal 0 1.1*max(u2_deg_sec)])
axis([0 5 0 0.015])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:0.005:0.015])
text(0.45,0.007,'u_2 - Multirate RK-4/RK-4','FontSize',11)

subplot(3,2,5)
x3=x3(:,2);
plot(t,x3,'b','LineWidth',1.3)
grid on
% axis([0 tfinal 1.2*min(x3) 1.1*max(x3)])
axis([0 5 -5e-4 10e-4])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[-5e-4:5e-4:15e-4])
text(1.0,4.0e-4,'x_3 - Simulink RK-4','FontSize',11)
xlabel('t  (s)','FontSize',12)

subplot(3,2,6)
plot(ts,u3,'b','LineWidth',1.3)
grid on
% axis([0 tfinal 1.2*min(u3) 1.1*max(u3)])
axis([0 5 -5e-4 10e-4])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[-5e-4:5e-4:15e-4])
text(0.45,5e-4,'u_3 - Multi-rate RK-4/RK-4','FontSize',11)
xlabel('t  (s)','FontSize',11)


% %%%%% Plot fast states w1,w2,w3,w4

% figure(3) % plot Figure 8.54
tf=linspace(0,tfinal,jcount);
% 
subplot(2,2,1)
t=x4(:,1);
x4=x4(:,2);
plot(t,x4,'b','LineWidth',1.3)
grid on
%axis([0 tfinal -0.24 1.25])
axis([0 tfinal 0 0.75])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:0.25:0.75])
text(1.0,0.53,'x_4 - Simulink RK-4','FontSize',12)
text(1.9,0.85,'Simulink and Multi-rate Integration of Fast States','FontSize',11)

subplot(2,2,2)
w1_deg=(180/pi)*w1; % convert w1 from rad to degrees
plot(tf,w1_deg,'b','LineWidth',1.3)
grid on
axis([0 tfinal 0 0.75])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:0.25:0.75])
text(0.23,0.528,'w_1 - Multi-rate RK-4/RK-4','FontSize',11)

subplot(2,2,3)
t=x5(:,1);
x5=x5(:,2);
plot(t,x5,'b','LineWidth',1.3)
grid on
axis([0 1 -0.2 0.2])
set(gca,'FontSize',11,'XTick',[0:0.25:1],'YTick',[-0.2:0.1:0.2])
text(0.25,0.113,'x_5 - Simulink RK-4','FontSize',11)
xlabel('t  (s)','FontSize',11)

subplot(2,2,4)
plot(tf,w2,'b','LineWidth',1.3)
grid on
axis([0 1 -0.2 0.2])
set(gca,'FontSize',11,'XTick',[0:0.25:1],'YTick',[-0.2:0.1:0.2])
text(0.075,0.115,'w_2 - Multi-rate RK-4/RK-4','FontSize',11)
xlabel('t  (s)','FontSize',11)

figure(4) % plot of x3,x4,w3,w4 not in text
subplot(2,2,1)
t=x6(:,1);
x6=x6(:,2);
plot(t,x6)
axis([-0.05 1 -19 20])
text(0.25,10,'\itx_{\rm6} \rm- Simulink RK-4','FontSize',11)

subplot(2,2,2)
plot(tf,w3)
axis([-0.05 1 -19 20])
text(0.10,12,'\itw_{\rm3} \rm- Multi-rate RK-4/RK-4','FontSize',11)

subplot(2,2,3)
t=x7(:,1);
x7=x7(:,2);
plot(t,x7)
xlabel('\itt \rm(sec)','FontSize',11)
axis([-0.05 1 -1.2e-4 1.1e-4])
text(0.20,7e-5,'\itx_{\rm7} \rm- Simulink RK-4','FontSize',11)

subplot(2,2,4)
plot(tf,w4)
xlabel('\itt \rm(sec)','FontSize',11)
axis([-0.05 1 -1.2e-4 1.1e-4])
text(0.10,7e-5,'\itw_{\rm4} \rm- Multi-rate RK-4/RK-4','FontSize',11)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Parallel realization of closed-loop transfer function %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Following results holds only for baseline system parameter values

d0=wnsq*Kc*z*(1+K*KA*Kq);
d1=Kc*(2*zeta*wn*z+wnsq+K*KA*Kq*wnsq*(1+tauA*z));
d2=Kc*z+2*zeta*wn*Kc+wnsq*p1*p2+K*Kc*KA*Kq*tauA*wnsq;
d3=Kc+2*zeta*wn*p1*p2+wnsq*(p1+p2);
d4=p1*p2+2*zeta*wn*(p1+p2)+wnsq;
d5=p1+p2+2*zeta*wn;
n0=K*Kc*KA*wnsq*z;
n1=K*Kc*KA*(1+tauA*z)*wnsq;
n2=K*Kc*KA*wnsq*tauA;
beta0=n0; beta1=n1; beta2=n2; % numerator coefficients of closed loop transfer function
num_xfer=[beta2 beta1 beta0];
alpha0=n0;
alpha1=n1+d0;
alpha2=n2+d1;
alpha3=d2;
alpha4=d3;
alpha5=d4;
alpha6=d5;
char_poly=[1 alpha6 alpha5 alpha4 alpha3 alpha2 alpha1 alpha0]; % 7th order polynomial in s
roots(char_poly); eig(A); % check roots of characteristic polynomial against eigenvalues of matrix A
[R,P]=residue(num_xfer,char_poly); % partial fraction expansion of closed loop transfer function
  
for i=1:7
    PP(i)=P(8-i); % reorder poles from smallest to largest magnitude
    RR(i)=R(8-i); % reorder residues to agree with reordered poles
end
R1=RR(1);R2=RR(2);R4=RR(4);R5=RR(5);R7=RR(7); 
p_1=PP(1); p_2=PP(2); p_4=PP(4); p_5=PP(5); p_7=PP(7); 
lam1=PP(1); lam2=PP(2); lam4=PP(4); lam7=PP(7);
A1=R1; 
a=real(R2);b=imag(R2);c=real(p_2);d=imag(p_2);
B1=2*a; B0=-2*(a*c+b*d); b1=-2*c; b0=c^2 + d^2;
wn_slow=sqrt(b0); zeta_slow=0.5*b1/wn_slow;  wd_slow= sqrt(1-zeta_slow^2)*wn_slow; % compute slow 2nd order component parameters
e_zeta_slow_coef=(1/30)*(zeta_slow-3*zeta_slow^3+2*zeta_slow^5)*(wn_slow^4); % coeff of T^4 term in damping ratio characteristic error
e_zeta_slow_des=0.01; % design value for damping ratio characteristic error
T_slow_zeta=(e_zeta_slow_des/e_zeta_slow_coef)^0.25; % slow system step size
e_wn_slow_coef=(1/120)*(1-8*zeta_slow^2+8*zeta_slow^4)*(wn_slow^4); % % coeff of T^4 term in natural frequency characteristic error
e_wd_slow_coef=(1/120)*(1-12*zeta_slow^2+16*zeta_slow^4)*(wn_slow^4); % % coeff of T^4 term in damping nat freq characteristic error
e_wd_slow_des=-0.00025; % design value for damped natural frequency characteristic error
T_slow_wd=(e_wd_slow_des/e_wd_slow_coef)^0.25;   % slow system step size
asym_check=wn_slow*T_slow_wd;   % check that wnT<<1

C1=R4;
a=real(R5);b=imag(R5);c=real(p_5);d=imag(p_5);
D1=2*a; D0=-2*(a*c+b*d); d1=-2*c; d0=c^2 + d^2;
wn_fast=sqrt(d0); zeta_fast=0.5*d1/wn_fast;  wd_fast= sqrt(1-zeta_fast^2)*wn_fast; % compute fast 2nd order component parameters
E1=R7;
E_des=-0.0002; 
T_fast=(-120*E_des/(lam4^4 + lam7^4))^0.25;  
asym_check=wn_fast*T_fast;   % check that wnT<<1

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%% Analysis of x1=theta %%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Begin Analytical Solution for theta 
% Following results holds only for baseline system parameter values

denom_theta=conv([tau_com 1 0],char_poly); % 9th order polynomial in s
num_theta=[n2 n1 n0];
[R,P]=residue(num_theta,denom_theta);  

lam1=P(8); lam2=P(7); lam3=P(6); % slow eigenvalues (lam1 is real, lam2,lam3 are complex) 
lam4=P(5); lam5=P(4); lam6=P(3); lam7=P(1); % fast eigenvalues  (lam4,lam7 are real, lam5,lam6 are complex)
p_com=P(2); % pole for command input
R1=R(8); R2=R(7); R3=R(6); % slow residues (R1 is real, R2,R3 are complex)
R4=R(5); R5=R(4); R6=R(3); R7=R(1); % fast residues (R4,R7 are real, R5,R6 are complex)
R_com=R(2); % residue for command input

ti=linspace(0,tfinal,2000);

% slow component of theta
a2=real(R2); b2=imag(R2);
c2=real(lam2); d2=imag(lam2);
theta_slow=theta_com_amp_deg*(R1*exp(lam1*ti)+2*exp(c2*ti).*(a2*cos(d2*ti)-b2*sin(d2*ti)));

% fast component of theta
a5=real(R5); b5=imag(R5);
c5=real(lam5); d5=imag(lam5);
theta_fast=theta_com_amp_deg*(R4*exp(lam4*ti)+R7*exp(lam7*ti)+2*exp(c5*ti).*(a5*cos(d5*ti)-b5*sin(d5*ti)));

% forced component of theta
theta_forced=theta_com_amp_deg*(R_com*exp(p_com*ti)+1);

theta=theta_slow+theta_fast+theta_forced;

figure(4) % plot Figure 8.52
plot(ti,theta,'b','LineWidth',1.3)
grid on
hold on
axis([0 tfinal 0 theta_com_amp_deg])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:5])

incr=2;
end1=20;
plot(t(1:incr:end1),x1(1:incr:end1),'k*','MarkerSize',6)
incr=10;
plot(t(end1:incr:end),x1(end1:incr:end),'k*','MarkerSize',6)

incr=1;
end1=15;
plot(ts(1:incr:end1),u1_deg(1:incr:end1),'r.','MarkerSize',10)
incr=4;
plot(ts(1:incr:end),u1_deg(1:incr:end),'r.','MarkerSize',10)

text(1,2.8,'Analytical','FontSize',11)
plot([0.55,0.85],[2.8,2.8],'-b','LineWidth',1.3)
text(1,2.3,'Simulink RK-4 (T = 0.0025 s)','FontSize',12)
plot(0.7,2.3,'k*','MarkerSize',6)
text(1,1.75,'Multirate RK-4/RK-4 (T_f = 0.0025 s, T_s = 0.05 s)','FontSize',12)
plot(0.7,1.75,'r.','MarkerSize',10)
text(1,1.10,'Frame Ratio \itN \rm = 20','FontSize',12)
xlabel('t  (s)','FontSize',13)
text(2.2,5.2,'x_1(t) vs t','FontSize',12)
ylabel('x_1  (deg)','FontSize',13)

% Begin plot components of analytical solution for theta
figure(5) % plot Figure 8.49
THETA_slow=5*(-0.5047*exp(-0.673*ti)+exp(-4.313*ti).*(-0.615*cos(6.4812*ti)-0.3474*sin(6.4812*ti)));
THETA_fast=5*(0.0004548*exp(-148.589*ti)+0.1025*exp(-21.975*ti)+exp(-11.069*ti).*(0.02045*cos(37.4583*ti)+0.0388*sin(37.4583*ti)));
THETA_forced=5*(1-0.0035*exp(-100*ti));
THETA=THETA_slow+THETA_fast+THETA_forced;

subplot(2,1,1)
hold on
plot(ti,theta_slow,'b','LineWidth',1.3)
grid on
axis([0 5 -6 2])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[-6:2:2])
plot(ti,theta_fast,'r','LineWidth',1.3)
text(1,1,'\theta_{fast}(t)','FontSize',12)
text(0.4,-3.65,'\theta_{slow}(t)','FontSize',12)
text(0.45,2.75,'Slow and Fast Components of Pitch Response \theta(t)','FontSize',12)
ylabel('\theta_{fast}   and   \theta_{slow}   (deg)','FontSize',12)

subplot(2,1,2)
hold on
plot(ti,theta_forced,'b','LineWidth',1.3)
grid on
axis([0 5 -0.5 6.5])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:2:6])
plot(ti,theta,'r','LineWidth',1.3)
text(1,5.8,'\theta_{forced}(t)','FontSize',12)
text(0.4,2.35,'\theta(t)','FontSize',12)
text(0.65,7.2,'Forced Component and Total Pitch Response','FontSize',12)
xlabel('t  (s)','FontSize',12)
ylabel('\theta_{forced}   and   \theta   (deg)','FontSize',12)

% % Begin Fourier analysis of theta_slow and theta_fast
% figure(6)
% wmin=0.01; wmax=10; % rad/sec
% lim1=log10(wmin); lim2=log10(wmax);
% w=logspace(lim1,lim2,500);
% s=j*w;
% F_slow=theta_com_amp_deg*(R1./(s-lam1)+R2./(s-lam2)+R3./(s-lam3));
% magF_slow=abs(F_slow);
% subplot(2,1,1)
% semilogx(w,magF_slow)
% ylabel('|\theta_{slow}(\itj\omega\rm)|','FontSize',11)
% title('Fourier Spectrum of \theta_{slow}\rm(\itt\rm)','FontSize',11)
% wmin=0.01; wmax=10; % rad/sec
% lim1=log10(wmin); lim2=log10(wmax);
% w=logspace(lim1,lim2,500);
% s=j*w;
% F_fast=theta_com_amp_deg*(R4./(s-lam4)+R5./(s-lam5)+R6./(s-lam6)+R7./(s-lam7));
% magF_fast=abs(F_fast);
% subplot(2,1,2)
% semilogx(w,magF_fast)
% ylabel('|\theta_{fast}(\itj\omega\rm)|','FontSize',11)
% xlabel('\omega (rad/sec)','FontSize',11)
% title('Fourier Spectrum of \theta_{fast}\rm(\itt\rm)','FontSize',11)
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%% Analysis of x2 %%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Begin analytical solution for x2
% Following results hold only for baseline system parameter values

% factor out s from numerator and denominator
denom_x2_anal=conv([tau_com 1],char_poly); % 8th order polynomial in s
num_x2_anal=K*Kc*[1 z]; % 1st order polynomial in s
[R,P]=residue(num_x2_anal,denom_x2_anal);  
R1=R(1);R2=R(2);R3=R(3);R5=R(5);R6=R(6);R8=R(8);
p_1=P(1);p_2=P(2);p_3=P(3);p_5=P(5);p_6=P(6);p_8=P(8);
a3=real(R3); b3=imag(R3);
c3=real(p_3); d3=imag(p_3);
a6=real(R6); b6=imag(R6);
c6=real(p_6); d6=imag(p_6);
x2_anal_1=R1*exp(p_1*tf)+R2*exp(p_2*tf)+R5*exp(p_5*tf)+R8*exp(p_8*tf); 
x2_anal_3=2*exp(c3*tf).*(a3*cos(d3*tf)-b3*sin(d3*tf));
x2_anal_6=2*exp(c6*tf).*(a6*cos(d6*tf)-b6*sin(d6*tf));
x2_anal=theta_com_amp_deg*(x2_anal_1+x2_anal_3+x2_anal_6);

figure(7) % plot Figure 8.53
plot(tf,x2_anal,'b','LineWidth',1.3)
grid on
axis([0 tfinal 0 0.014])
set(gca,'FontSize',11,'XTick',[0:5],'YTick',[0:0.002:0.014])
hold on

incr1=1;
end1=23;
plot(t(1:incr1:end1),x2(1:incr1:end1),'k*','MarkerSize',6)
incr2=5;
end2=100;
plot(t(end1:incr2:end2),x2(end1:incr2:end2),'k*','MarkerSize',6)
incr3=10;
plot(t(end2:incr3:end),x2(end2:incr3:end),'k*','MarkerSize',6)

incr1=1;
end1=20;
plot(ts(1:incr1:end1),u2_deg_sec(1:incr1:end1),'r.','MarkerSize',10)
incr2=4;
plot(ts(end1:incr2:end),u2_deg_sec(end1:incr2:end),'r.','MarkerSize',10)


text(1.5,0.0098,'Analytical','FontSize',13)
plot([1.2,1.4],[0.0097,0.0097],'-b','LineWidth',1.3)

text(1.50,0.0115,'Simulink RK-4 (T = 0.0025 s)','FontSize',13)
plot(1.3,0.0115,'k*','MarkerSize',6)

text(0.80,0.013,'Multirate RK-4/RK-4 (T_f = 0.0025 s, T_s = 0.05 s)','FontSize',13)
plot(0.7,0.013,'r.','MarkerSize',10)

text(2.5,0.007,'Frame Ratio N  = 20','FontSize',13)
xlabel('t  (s)','FontSize',13)
ylabel('x_2','FontSize',13)
text(2,0.01465,'x_2 vs. t','FontSize',13)

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%% Analysis of x4=delta_e %%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Begin analytical solution for delta_e
% Following results holds only for baseline system parameter values

denom_delta_e=conv([tau_com 1],char_poly); % 8th order polynomial in s
m0=wnsq*z;
m1=wnsq+2*zeta*wn*z;
m2=2*zeta*wn+z;
m3=1;
num_delta_e=K*Kc*[m3 m2 m1 m0];
[R,P]=residue(num_delta_e,denom_delta_e);  

% lam1=P(8); lam2=P(7); lam3=P(6); slow eigenvalues (lam1 is real, lam2,lam3 are complex) 
% lam4=P(5); lam5=P(4); lam6=P(3); lam7=P(1); fast eigenvalues  (lam4,lam7 are real, lam5,lam6 are complex)
% p_com=P(2); pole for command input

R1=R(8); R2=R(7); R3=R(6); % slow residues (R1 is real, R2,R3 are complex)
R4=R(5); R5=R(4); R6=R(3); R7=R(1); % fast residues (R4,R7 are real, R5,R6 are complex)
R_com=R(2); % residue for command input

% slow component of delta_e
a2=real(R2); b2=imag(R2);
c2=real(lam2); d2=imag(lam2);
delta_e_slow=theta_com_amp_deg*(R1*exp(lam1*ti)+2*exp(c2*ti).*(a2*cos(d2*ti)-b2*sin(d2*ti)));

% fast component of delta_e
a5=real(R5); b5=imag(R5);
c5=real(lam5); d5=imag(lam5);
delta_e_fast=theta_com_amp_deg*(R4*exp(lam4*ti)+R7*exp(lam7*ti)+2*exp(c5*ti).*(a5*cos(d5*ti)-b5*sin(d5*ti)));

% forced component of delta_e
delta_e_forced=theta_com_amp_deg*(R_com*exp(p_com*ti));

delta_e=delta_e_slow+delta_e_fast+delta_e_forced;

% figure(8) % plot Figure 8.55
% plot(ti,delta_e,'b','LineWidth',1.3)
% grid on
% hold on
% axis([0 1 -0.2 1.2])
% set(gca,'FontSize',11,'XTick',[0:0.21],'YTick',[-0.2:0.2:1.2])
% 
%  x4=x4(:,2);
% incr1=1;
% end1=20;
% plot(t(1:incr1:end1),x4(1:incr1:end1),'k*','MarkerSize',6)
% incr2=2;
% end2=100;
% plot(t(end1:incr2:end2),x4(end1:incr2:end2),'k*','MarkerSize',6)
%  
% incr1=1;
% end1=25;
% plot(tf(1:incr1:end),w1_deg(1:incr1:end),'r.','MarkerSize',10)
% % plot([0.13,0.16],[0.9,0.9],'-k')
% 
% 
% text(0.17,0.9,'Analytical','FontSize',11)
% plot(0.15,0.75,'r*','MarkerSize',5)
% text(0.17,0.75,'Simulink RK-4 (T = 0.0025 s)','FontSize',13)
% plot(0.15,0.6,'r.','MarkerSize',10)
% text(0.17,0.6,'Multi-rate RK-4/RK-4 (T_f = 0.0025 s, T_s = 0.05 s)','FontSize',13)
% text(0.17,0.4,'Frame Ratio N = 20','FontSize',13)
% xlabel('t  (s)','FontSize',13)
% title('x_4(t) vs. t','FontSize',11)
% ylabel('x_4 (deg)','FontSize',13)

% plot components of elevator deflection response delta_e
figure(9)
% delta_e_slow=5*(0.0709*exp(-0.673*tf)+exp(-4.313*tf).*(0.0844*cos(6.4812*tf)-0.1440*sin(6.4812*tf)));
% delta_e_fast=5*(0.050*exp(-148.589*tf)+0.252*exp(-21.975*tf)+exp(-11.069*tf).*(-0.2842*cos(37.4583*tf)-0.1644*sin(37.4583*tf)));
% delta_e_forced=5*(-0.1733*exp(-100*tf));
% delta_e=delta_e_slow+delta_e_fast+delta_e_forced;
subplot(2,1,1)
hold on
plot(ti,delta_e_slow,'b')
plot(ti,delta_e_fast,'r')
axis([-0.2 5 -0.6 1])
text(0.75,0.4,'(\delta_{e})_{\rmslow}(\itt\rm)','FontSize',11)
text(0.1,-0.4,'(\delta_{e})_{\rmfast}(\itt\rm)','FontSize',11)
title('Slow and Fast Components of Elevator Deflection Response \delta_{\rme}\rm(\itt\rm)','FontSize',11)
ylabel('deg','FontSize',11)
subplot(2,1,2)
hold on
plot(ti,delta_e_forced,'g')
plot(ti,delta_e,'k')
axis([-0.2 5 -1.2 1.2])
text(0.1,-0.7,'(\delta_{e})_{\rmforced}(\itt\rm)','FontSize',11)
text(0.2,0.75,'(\delta_{e})(\itt\rm)','FontSize',11)
title('Forced Component and Total Pitch Response','FontSize',11)
xlabel('\itt \rm(sec)','FontSize',11)
ylabel('deg','FontSize',11)
title('Forced Component and Total Elevator Deflection Response \delta_{\rme}\rm(\itt\rm)','FontSize',11)

% % Begin Fourier analysis of delta_e_slow and delta_e_fast
% figure(10)
% wmin=0.01; wmax=10; % rad/sec
% lim1=log10(wmin); lim2=log10(wmax);
% w=logspace(lim1,lim2,500);
% s=j*w;
% F_delta_e_slow=theta_com_amp_deg*(R1./(s-lam1)+R2./(s-lam2)+R3./(s-lam3));
% magF_delta_e_slow=abs(F_delta_e_slow);
% subplot(2,1,1)
% semilogx(w,magF_delta_e_slow)
% ylabel('|(\delta_{e})_{slow}(\itj\omega\rm)|','FontSize',11)
% title('Fourier Spectrum of (\delta_{e})_{slow}\rm(\itt\rm)','FontSize',11)
% wmin=0.01; wmax=100; % rad/sec
% lim1=log10(wmin); lim2=log10(wmax);
% w=logspace(lim1,lim2,500);
% s=j*w;
% F_delta_e_fast=theta_com_amp_deg*(R4./(s-lam4)+R5./(s-lam5)+R6./(s-lam6)+R7./(s-lam7));
% magF_delta_e_fast=abs(F_delta_e_fast);
% subplot(2,1,2)
% semilogx(w,magF_delta_e_fast)
% ylabel('|(\delta_{e})_{fast}(\itj\omega\rm)|','FontSize',11)
% xlabel('\omega (rad/sec)','FontSize',11)
% title('Fourier Spectrum of (\delta_{e})_{fast}\rm(\itt\rm)','FontSize',11)
% 
% 
