% m-file Ch8_Sec4_TwoTanks_Multirate_AB2.m
% calls Simulink model TwoTanks.mdl
% plots Figure 8.63 and Figure 8.64

close all, clear all, clc

global A1 A2 L1 L2 c1 c2 gam p0 

% baseline parameter values
gam=62.4; % specific wt of water (lb per cu ft) 
p0=14.7*144; % atm pressure (lb/sq ft)
R1=15; % ft
A1=pi*R1^2;
L1=25; % ft
R2=5; % ft
A2=pi*R2^2;
L2=7.5; % ft
c1=4; % cu ft/min per psf^1/2
c2=2; % 
EPS=1e-16;  % tolerance for flow calculations
F0=60;

%%%%% Begin AB/2 - AB/2 multi-rate integration of two tanks model equations
%% slow state: H1
%% fast state: H2

Ts=25; % slow time frame for Figure 4.25
Tf=0.25; % fast time frame
N=round(Ts/Tf); % frame ratio
tfinal=1600; % final simulation time (sec)
ifinal=round(tfinal/Ts); % total number of slow time frames
H1_init=0; % initialize slow state
H1(1)=H1_init; % H1 is slow state array for plotting
ts(1)=0;
H2_init=0; % initialize fast state
H2(1)=H2_init; % H2 is fast state array for plotting
tf(1)=0;

%%% Step 1 - Take Euler step to find H1(Ts) and H2(Tf)
k11=deriv_H1(H1_init,H2_init,F0); 

H1(2)=H1_init+Ts*k11;  % compute H1(Ts) using RK-1
ts(2)=Ts;
H2(2)=H2_init+Tf*deriv_H2(H1_init,H2_init,F0);  % compute H2(Tf) using RK-1
tf(2)=Tf;

%%% Step 2 - Advance H2 from H2(Tf) to H2(Ts)=H2(NTf) using AB-2 with
%%% interpolated values of H1
j=2; % initialze index for tf and H2 array

for k=2:N 
   j=j+1; % index for H2 and tf arrays
   tk_past=(k-2)*Tf; 
   tk_current=(k-1)*Tf;  
   H1_past=H1_init+k11*tk_past;  % interpolated value of past H1 required to update H2
   H1_current=H1_init+k11*tk_current;  % interpolated value of current H1 required to update H2
   H2_past=H2(k-1);  % past value of H2
   H2_current=H2(k);  % current value of H2 
   H2_new=H2_current+(0.5*Tf)*(3*deriv_H2(H1_current,H2_current,F0)-deriv_H2(H1_past,H2_past,F0));  % updated H2 
   H2(j)=H2_new;  % save updated H2
   tf(j)=k*Tf; % save new tf
end
   
%%% Step 3 - Begin outer loop to advance H1 by 1 slow time frame Ts using AB-2

ifinal=round(tfinal/Ts); % compute total number of slow frames

for i=2:ifinal   % start from beginning of 2nd slow frame  
   H1_past=H1(i-1); 
   H1_current=H1(i); 
   H2_past=H2(j-N); 
   H2_current=H2(j); 
   H1_new=H1_current+(0.5*Ts)*(3*deriv_H1(H1_current,H2_current,F0)-deriv_H1(H1_past,H2_past,F0));    
   H1(i+1)=H1_new ;
   ts(i+1)=i*Ts ;   

% %%% Step 4 - Begin inner loop to integrate H2 N fast time frames during each slow time frame
slope_past=(H1(i)-H1(i-1))/Ts;   % slope of line connecting past H1 and current H1 values
slope_next=(H1(i+1)-H1(i))/Ts;  % slope of line connecting current and new H1 values
  
for k=1:N    
    if k==1 % starting point for fast integration loop
        H1_past=H1(i)-slope_past*Tf;
        H1_current=H1(i);
    else
        H1_past=H1(i)+slope_next*(k-2)*Tf;
        H1_current=H1(i)+slope_next*(k-1)*Tf;
    end
    
H2_past=H2(j-1);
H2_current=H2(j);
H2_new=H2_current+(0.5*Tf)*(3*deriv_H2(H1_current,H2_current,F0)-deriv_H2(H1_past,H2_past,F0));
H2(j+1)=H2_new;
tf(j+1)=j*Tf;
j=j+1;
end % k
 
end % i


T=0.1; % sim step size for SIMULINK RK-4 simulation
sim('TwoTanks')
t=H_1(:,1);
H1_sim=H_1(:,2);
H2_sim=H_2(:,2);


figure(1) % plot Figure 8.63
subplot(2,1,1)
plot(t,H1_sim,'b','LineWidth',1.3)
grid on
axis([0 1600 0 25])
set(gca,'FontSize',11,'XTick',[0:200:1600],'YTick',[0:5:25])
hold on

incr1=1;
end1=5;
plot(ts(1:incr1:end1),H1(1:incr1:end1),'.r','MarkerSize',9)
incr2=2;
plot(ts(end1:incr2:end),H1(end1:incr2:end),'.r','MarkerSize',9)

ylabel('H_1  (ft)','FontSize',13)
text(400,27,'Simulated Tank 1 Level vs. Time','FontSize',12)
text(350,9,'SIMULINK RK-4 (T = 0.1 min)','FontSize',11)
plot([275 325],[9 9],'b','LineWidth',1.3)
text(185,4,'Multirate integration AB-2/AB-2 (\itT_{\rms}\rm=25 min,\itT_{\rmf}\rm=0.25 min)','FontSize',11)
plot(160,4,'r.','MarkerSize',9)

subplot(2,1,2)
plot(t,H2_sim,'b','LineWidth',1.3)
grid on
hold on
axis([0 1600 0 2.25])
set(gca,'FontSize',11,'XTick',[0:200:1600],'YTick',[0:0.5:2])

incr1=100;
end1=2000;
plot(tf(1:incr1:end1),H2(1:incr1:end1),'r.','MarkerSize',9)
incr2=125;
plot(tf(end1:incr2:end),H2(end1:incr2:end),'r.','MarkerSize',9)


xlabel('t  (min)','FontSize',13)
ylabel('H_2(t)  ft','FontSize',13)
text(400,2.40,'Simulated Tank 2 Level vs. Time','FontSize',12)

text(350,0.75,'SIMULINK RK-4 (T = 0.1 min)','FontSize',11)
plot([275 325],[0.75 0.75],'b','LineWidth',1.3)

text(185,0.25,'Multirate integration AB-2/AB-2 (T_s=25 min,T_f=0.25 min)','FontSize',11)
plot(160,0.30,'r.','MarkerSize',9)


figure(2) % plot Figure 8.64
clear ts  tf H1 H2
Ts=100; % slow time frame for Figure 4.26
Tf=0.1; % fast time frame
N=round(Ts/Tf); % frame ratio
tfinal=1600; % final simulation time (sec)
ifinal=round(tfinal/Ts); % total number of slow time frames
H1_init=0; % initialize slow state
H1(1)=H1_init; % H1 is slow state array for plotting
ts(1)=0;
H2_init=0; % initialize fast state
H2(1)=H2_init; % H2 is fast state array for plotting
tf(1)=0;

%%% Step 1 - Take Euler step to find H1(Ts) and H2(Tf)
k11=deriv_H1(H1_init,H2_init,F0); 

H1(2)=H1_init+Ts*k11;  % compute H1(Ts) using RK-1
ts(2)=Ts;
H2(2)=H2_init+Tf*deriv_H2(H1_init,H2_init,F0);  % compute H2(Tf) using RK-1
tf(2)=Tf;

%%% Step 2 - Advance H2 from H2(Tf) to H2(Ts)=H2(NTf) using AB-2 with
%%% interpolated values of H1
j=2; % initialze index for tf and H2 array

for k=2:N 
   j=j+1; % index for H2 and tf arrays
   tk_past=(k-2)*Tf; 
   tk_current=(k-1)*Tf;  
   H1_past=H1_init+k11*tk_past;  % interpolated value of past H1 required to update H2
   H1_current=H1_init+k11*tk_current;  % interpolated value of current H1 required to update H2
   H2_past=H2(k-1);  % past value of H2
   H2_current=H2(k);  % current value of H2 
   H2_new=H2_current+(0.5*Tf)*(3*deriv_H2(H1_current,H2_current,F0)-deriv_H2(H1_past,H2_past,F0));  % updated H2 
   H2(j)=H2_new;  % save updated H2
   tf(j)=k*Tf; % save new tf
end
   
%%% Step 3 - Begin outer loop to advance H1 by 1 slow time frame Ts using AB-2

ifinal=round(tfinal/Ts); % compute total number of slow frames

for i=2:ifinal   % start from beginning of 2nd slow frame  
   H1_past=H1(i-1); 
   H1_current=H1(i); 
   H2_past=H2(j-N); 
   H2_current=H2(j); 
   H1_new=H1_current+(0.5*Ts)*(3*deriv_H1(H1_current,H2_current,F0)-deriv_H1(H1_past,H2_past,F0));    
   H1(i+1)=H1_new ;
   ts(i+1)=i*Ts ;   

% %%% Step 4 - Begin inner loop to integrate H2 N fast time frames during each slow time frame
slope_past=(H1(i)-H1(i-1))/Ts;   % slope of line connecting past H1 and current H1 values
slope_next=(H1(i+1)-H1(i))/Ts;  % slope of line connecting current and new H1 values
  
for k=1:N    
    if k==1 % starting point for fast integration loop
        H1_past=H1(i)-slope_past*Tf;
        H1_current=H1(i);
    else
        H1_past=H1(i)+slope_next*(k-2)*Tf;
        H1_current=H1(i)+slope_next*(k-1)*Tf;
    end
    
H2_past=H2(j-1);
H2_current=H2(j);
H2_new=H2_current+(0.5*Tf)*(3*deriv_H2(H1_current,H2_current,F0)-deriv_H2(H1_past,H2_past,F0));
H2(j+1)=H2_new;
tf(j+1)=j*Tf;
j=j+1;
end % k
 
end % i

T=0.1; % sim step size for SIMULINK RK-4 simulation
sim('TwoTanks')
t=H_1(:,1);
H1_sim=H_1(:,2);
H2_sim=H_2(:,2);
    
subplot(2,1,1)
plot(t,H1_sim,'b','LineWidth',1.3)
grid on
axis([0 1600 0 25])
set(gca,'FontSize',11,'XTick',[0:200:1600],'YTick',[0:5:25])
hold on

plot(ts(1:end),H1(1:end),'.r','MarkerSize',9)

ylabel('H_1(t)   (ft)','FontSize',13)
text(400,27,'Simulated Tank 1 Level vs. Time','FontSize',12)
text(400,11.5,'SIMULINK RK-4 (T = 0.1 min)','FontSize',12)
plot([300 375],[11.2 11.2],'b','LineWidth',1.3)
text(200,4,'Multirate integration AB-2/AB-2 (T_s=100 min,T_f=0.1 min)','FontSize',12)
plot(150,4.1,'r.','MarkerSize',9)

subplot(2,1,2)
plot(t,H2_sim,'b','LineWidth',1.3)
grid on
axis([0 1600 0 2.2])
set(gca,'FontSize',11,'XTick',[0:200:1600],'YTick',[0:0.5:2])
hold on
incr1=500;
plot(tf(1:incr1:end),H2(1:incr1:end),'.r','MarkerSize',9)

xlabel('t  (min)','FontSize',13)
ylabel('H_2(ft)   (ft)','FontSize',13)
text(400,2.4,'Simulated Tank 2 Level vs Time','FontSize',12)  
text(600,0.9,'SIMULINK RK-4 (T = 0.1 min)','FontSize',12)
plot([490 565],[0.87 0.87],'b','LineWidth',1.3)
text(250,0.25,'Multirate integration AB-2/AB-2 (T_s=100 min,T_f=0.1 min)','FontSize',12)
plot(210,0.28,'r.','MarkerSize',9)
